@@ -15,14 +15,12 @@ require 'tmpdir'
 begin
   require 'kramdown/converter/syntax_highlighter/rouge'
 
-  class Kramdown::Converter::SyntaxHighlighter::Rouge::FORMATTER_CLASS
-    def format(tokens, &b)
-      super.sub(/<\/code><\/pre>\n?/, "</code></pre>\n")
-    end
+  Kramdown::Converter::SyntaxHighlighter::Rouge.formatter_class.send(:define_method, :format) do |tokens, &b|
+    super(tokens, &b).sub(/<\/code><\/pre>\n?/, "</code></pre>\n")
   end
 
   # custom formatter for tests
-  class RougeHTMLFormatters < Kramdown::Converter::SyntaxHighlighter::Rouge::FORMATTER_CLASS
+  class RougeHTMLFormatters < Kramdown::Converter::SyntaxHighlighter::Rouge.formatter_class
     tag 'rouge_html_formatters'
 
     def stream(tokens, &b)
@@ -38,11 +36,35 @@ Encoding.default_external = 'utf-8' if RUBY_VERSION >= '1.9'
 
 class TestFiles < Minitest::Test
 
+  MATHJAX_NODE_AVAILABLE = begin
+                             require 'kramdown/converter/math_engine/mathjaxnode'
+                             Kramdown::Converter::MathEngine::MathjaxNode::AVAILABLE or
+                               warn("Skipping MathjaxNode tests as MathjaxNode is not available")
+                           end
+
+  SSKATEX_AVAILABLE = begin
+                        / class="katex"/ === Kramdown::Document.
+                          new('$$a$$', {:math_engine => :sskatex}).to_html or
+                          warn("Skipping SsKaTeX tests as SsKaTeX is not available.")
+                      rescue
+                        warn("Skipping SsKaTeX tests as default SsKaTeX config does not work.")
+                      end or warn("Run \"rake dev:test_sskatex_deps\" to see why.")
+
+  KATEX_AVAILABLE = RUBY_VERSION >= '2.3'
+
   EXCLUDE_KD_FILES = [('test/testcases/block/04_header/with_auto_ids.text' if RUBY_VERSION <= '1.8.6'), # bc of dep stringex not working
                       ('test/testcases/span/03_codespan/rouge/' if RUBY_VERSION < '2.0'), #bc of rouge
                       ('test/testcases/block/06_codeblock/rouge/' if RUBY_VERSION < '2.0'), #bc of rouge
                       ('test/testcases/block/15_math/itex2mml.text' if RUBY_PLATFORM == 'java'), # bc of itextomml
                       ('test/testcases/span/math/itex2mml.text' if RUBY_PLATFORM == 'java'), # bc of itextomml
+                      ('test/testcases/block/15_math/mathjaxnode.text' unless MATHJAX_NODE_AVAILABLE),
+                      ('test/testcases/block/15_math/mathjaxnode_notexhints.text' unless MATHJAX_NODE_AVAILABLE),
+                      ('test/testcases/block/15_math/mathjaxnode_semantics.text' unless MATHJAX_NODE_AVAILABLE),
+                      ('test/testcases/span/math/mathjaxnode.text' unless MATHJAX_NODE_AVAILABLE),
+                      ('test/testcases/block/15_math/sskatex.text' unless SSKATEX_AVAILABLE),
+                      ('test/testcases/span/math/sskatex.text' unless SSKATEX_AVAILABLE),
+                      ('test/testcases/block/15_math/katex.text' unless KATEX_AVAILABLE),
+                      ('test/testcases/span/math/katex.text' unless KATEX_AVAILABLE),
                      ].compact
 
   # Generate test methods for kramdown-to-xxx conversion
@@ -78,6 +100,7 @@ class TestFiles < Minitest::Test
                           'test/testcases/block/06_codeblock/rouge/multiple.html', # bc of double surrounding <div>
                           ('test/testcases/span/03_codespan/rouge/simple.html' if RUBY_VERSION < '2.0'),
                           ('test/testcases/span/03_codespan/rouge/disabled.html' if RUBY_VERSION < '2.0'),
+                          'test/testcases/block/14_table/empty_tag_in_cell.html', # bc of tidy
                           'test/testcases/block/15_math/ritex.html', # bc of tidy
                           'test/testcases/span/math/ritex.html', # bc of tidy
                           'test/testcases/block/15_math/itex2mml.html', # bc of tidy
@@ -89,6 +112,10 @@ class TestFiles < Minitest::Test
                           'test/testcases/block/15_math/mathjax_preview.html', # bc of mathjax preview
                           'test/testcases/block/15_math/mathjax_preview_simple.html', # bc of mathjax preview
                           'test/testcases/block/15_math/mathjax_preview_as_code.html', # bc of mathjax preview
+                          'test/testcases/span/math/sskatex.html', # bc of tidy
+                          'test/testcases/block/15_math/sskatex.html', # bc of tidy
+                          'test/testcases/span/math/katex.html', # bc of tidy
+                          'test/testcases/block/15_math/katex.html', # bc of tidy
                           'test/testcases/span/05_html/mark_element.html', # bc of tidy
                           'test/testcases/block/09_html/xml.html', # bc of tidy
                           'test/testcases/span/05_html/xml.html', # bc of tidy
@@ -187,6 +214,7 @@ class TestFiles < Minitest::Test
                           ('test/testcases/span/03_codespan/rouge/disabled.text' if RUBY_VERSION < '2.0'), #bc of rouge
                           'test/testcases/block/06_codeblock/rouge/simple.text',
                           'test/testcases/block/06_codeblock/rouge/multiple.text', # check, what document contain more, than one code block
+                          'test/testcases/block/14_table/empty_tag_in_cell.text', # bc of tidy
                           'test/testcases/block/15_math/ritex.text', # bc of tidy
                           'test/testcases/span/math/ritex.text', # bc of tidy
                           'test/testcases/block/15_math/itex2mml.text', # bc of tidy
@@ -195,6 +223,10 @@ class TestFiles < Minitest::Test
                           'test/testcases/block/15_math/mathjaxnode_notexhints.text', # bc of tidy
                           'test/testcases/block/15_math/mathjaxnode_semantics.text', # bc of tidy
                           'test/testcases/span/math/mathjaxnode.text', # bc of tidy
+                          'test/testcases/block/15_math/sskatex.text', # bc of tidy
+                          'test/testcases/span/math/sskatex.text', # bc of tidy
+                          'test/testcases/block/15_math/katex.text', # bc of tidy
+                          'test/testcases/span/math/katex.text', # bc of tidy
                           'test/testcases/span/01_link/link_defs_with_ial.text', # bc of attribute ordering
                           'test/testcases/span/05_html/mark_element.text', # bc of tidy
                           'test/testcases/block/09_html/xml.text', # bc of tidy
@@ -238,6 +270,7 @@ class TestFiles < Minitest::Test
                              'test/testcases/span/autolinks/url_links.html',      # bc of quot entity being converted to char
                              ('test/testcases/span/03_codespan/rouge/simple.html' if RUBY_VERSION < '2.0'),
                              ('test/testcases/span/03_codespan/rouge/disabled.html' if RUBY_VERSION < '2.0'),
+                             'test/testcases/block/14_table/empty_tag_in_cell.html', # bc of tidy
                              'test/testcases/block/15_math/ritex.html', # bc of tidy
                              'test/testcases/span/math/ritex.html', # bc of tidy
                              'test/testcases/block/15_math/itex2mml.html', # bc of tidy
@@ -246,6 +279,10 @@ class TestFiles < Minitest::Test
                              'test/testcases/block/15_math/mathjaxnode_notexhints.html', # bc of tidy
                              'test/testcases/block/15_math/mathjaxnode_semantics.html', # bc of tidy
                              'test/testcases/span/math/mathjaxnode.html', # bc of tidy
+                             'test/testcases/block/15_math/sskatex.html', # bc of tidy
+                             'test/testcases/span/math/sskatex.html', # bc of tidy
+                             'test/testcases/block/15_math/katex.html', # bc of tidy
+                             'test/testcases/span/math/katex.html', # bc of tidy
                              'test/testcases/block/15_math/mathjax_preview.html', # bc of mathjax preview
                              'test/testcases/block/15_math/mathjax_preview_simple.html', # bc of mathjax preview
                              'test/testcases/block/15_math/mathjax_preview_as_code.html', # bc of mathjax preview
@@ -282,6 +319,7 @@ class TestFiles < Minitest::Test
                        'test/testcases/block/03_paragraph/no_newline_at_end.text',
                        'test/testcases/block/03_paragraph/indented.text',
                        'test/testcases/block/03_paragraph/two_para.text',
+                       'test/testcases/block/03_paragraph/line_break_last_line.text',
                        'test/testcases/block/04_header/atx_header.text',
                        'test/testcases/block/04_header/setext_header.text',
                        'test/testcases/block/04_header/with_auto_ids.text', # bc of ID generation difference
@@ -346,6 +384,14 @@ class TestFiles < Minitest::Test
                        ('test/testcases/block/06_codeblock/rouge/multiple.text' if RUBY_VERSION < '2.0'), #bc of rouge
                        ('test/testcases/block/15_math/itex2mml.text' if RUBY_PLATFORM == 'java'), # bc of itextomml
                        ('test/testcases/span/math/itex2mml.text' if RUBY_PLATFORM == 'java'), # bc of itextomml
+                       ('test/testcases/block/15_math/mathjaxnode.text' unless MATHJAX_NODE_AVAILABLE),
+                       ('test/testcases/block/15_math/mathjaxnode_notexhints.text' unless MATHJAX_NODE_AVAILABLE),
+                       ('test/testcases/block/15_math/mathjaxnode_semantics.text' unless MATHJAX_NODE_AVAILABLE),
+                       ('test/testcases/span/math/mathjaxnode.text' unless MATHJAX_NODE_AVAILABLE),
+                       ('test/testcases/block/15_math/sskatex.text' unless SSKATEX_AVAILABLE),
+                       ('test/testcases/span/math/sskatex.text' unless SSKATEX_AVAILABLE),
+                       ('test/testcases/block/15_math/katex.text' unless KATEX_AVAILABLE),
+                       ('test/testcases/span/math/katex.text' unless KATEX_AVAILABLE),
                       ].compact
 
   # Generate test methods for gfm-to-html conversion
@@ -377,7 +423,11 @@ class TestFiles < Minitest::Test
                        ].compact
 
   EXCLUDE_MODIFY = ['test/testcases/block/06_codeblock/rouge/multiple.text', # bc of HTMLFormater in options
-                   ]
+                    ('test/testcases/block/15_math/sskatex.text' unless SSKATEX_AVAILABLE),
+                    ('test/testcases/span/math/sskatex.text' unless SSKATEX_AVAILABLE),
+                    ('test/testcases/block/15_math/katex.text' unless KATEX_AVAILABLE),
+                    ('test/testcases/span/math/katex.text' unless KATEX_AVAILABLE),
+                   ].compact
 
   # Generate test methods for asserting that converters don't modify the document tree.
   Dir[File.dirname(__FILE__) + '/testcases/**/*.text'].each do |text_file|
